@@ -47,18 +47,18 @@ void TvSpielfilmFetcher::fetchGroup(const QString &url,
     m_provider.get(
         QUrl(url),
         [this, callback](const QByteArray &data) {
-            static QRegularExpression reChannelList(QStringLiteral("<select name=\\\"channel\\\">.*</select>"));
-            reChannelList.setPatternOptions(QRegularExpression::DotMatchesEverythingOption);
-            QRegularExpressionMatch matchChannelList = reChannelList.match(QString::fromUtf8(data));
+            static const QRegularExpression reChannelList(QStringLiteral("<select name=\\\"channel\\\">.*</select>"),
+                                                          QRegularExpression::DotMatchesEverythingOption);
+            const QRegularExpressionMatch matchChannelList = reChannelList.match(QString::fromUtf8(data));
             if (matchChannelList.hasMatch()) {
                 QMap<ChannelId, ChannelData> channels;
                 const QString channelList = matchChannelList.captured(0);
 
-                static QRegularExpression reChannel(QStringLiteral("<option.*?value=\\\"(.*?)\\\">&nbsp;&nbsp;(.*?)</option>"));
-                reChannel.setPatternOptions(QRegularExpression::DotMatchesEverythingOption);
+                static const QRegularExpression reChannel(QStringLiteral("<option.*?value=\\\"(.*?)\\\">&nbsp;&nbsp;(.*?)</option>"),
+                                                          QRegularExpression::DotMatchesEverythingOption);
                 QRegularExpressionMatchIterator it = reChannel.globalMatch(channelList);
                 while (it.hasNext()) {
-                    QRegularExpressionMatch channelMatch = it.next();
+                    const QRegularExpressionMatch channelMatch = it.next();
                     const ChannelId id = ChannelId(channelMatch.captured(1));
 
                     // exclude groups (e.g. "alle Sender" or "g:1")
@@ -160,13 +160,14 @@ void TvSpielfilmFetcher::fetchProgram(const ChannelId &channelId,
         QUrl(url),
         [this, channelId, date, page, programs, callback, errorCallback, url](const QByteArray &data) {
             QVector<ProgramData> allPrograms(programs);
-            allPrograms.append(processChannel(QString::fromUtf8(data), url, channelId));
+            processChannel(QString::fromUtf8(data), url, channelId, allPrograms);
 
             // fetch next page
-            static QRegularExpression reNextPage(QStringLiteral(
-                "<ul class=\\\"pagination__items\\\">.*</ul>\\s*<a href=\\\"(.*?)\\\".*class=\\\"js-track-link pagination__link pagination__link--next\\\""));
-            reNextPage.setPatternOptions(QRegularExpression::DotMatchesEverythingOption);
-            QRegularExpressionMatch matchNextPage = reNextPage.match(QString::fromUtf8(data));
+            static const QRegularExpression reNextPage(
+                QStringLiteral("<ul class=\\\"pagination__items\\\">.*</ul>\\s*<a href=\\\"(.*?)\\\".*class=\\\"js-track-link pagination__link "
+                               "pagination__link--next\\\""),
+                QRegularExpression::DotMatchesEverythingOption);
+            const QRegularExpressionMatch matchNextPage = reNextPage.match(QString::fromUtf8(data));
             if (matchNextPage.hasMatch()) {
                 fetchProgram(channelId, date, page + 1, allPrograms, callback, errorCallback);
             } else {
@@ -192,10 +193,8 @@ void TvSpielfilmFetcher::fetchProgram(const ChannelId &channelId,
         });
 }
 
-QVector<ProgramData> TvSpielfilmFetcher::processChannel(const QString &infoTable, const QString &url, const ChannelId &channelId)
+void TvSpielfilmFetcher::processChannel(const QString &infoTable, const QString &url, const ChannelId &channelId, QVector<ProgramData> &programs)
 {
-    QVector<ProgramData> programs;
-
     // column with title + description URL + start/stop time
     const QString reDescriptionUrl(QStringLiteral("<a href=\\\"(https://www.tvspielfilm.de/tv-programm/sendung/.*?\\.html)\\\""));
     const QString reTitle(QStringLiteral("<strong>(.*?)</strong>"));
@@ -207,19 +206,24 @@ QVector<ProgramData> TvSpielfilmFetcher::processChannel(const QString &infoTable
     const QString reCategory(QStringLiteral("<span>(.*?)</span>"));
     const QString reCategoryCol(QStringLiteral("<td class=\\\"col-4\\\">.*?") + reCategory + QStringLiteral(".*?</td>"));
 
-    QRegularExpression reProgram(QStringLiteral("<tr class=\\\"hover\\\">.*?") + reMainCol + QStringLiteral(".*?") + reCategoryCol
-                                 + QStringLiteral(".*?</tr>"));
-    reProgram.setPatternOptions(QRegularExpression::DotMatchesEverythingOption);
+    static const QRegularExpression reProgram(QStringLiteral("<tr class=\\\"hover\\\">.*?") + reMainCol + QStringLiteral(".*?") + reCategoryCol
+                                                  + QStringLiteral(".*?</tr>"),
+                                              QRegularExpression::DotMatchesEverythingOption);
     QRegularExpressionMatchIterator it = reProgram.globalMatch(infoTable);
     while (it.hasNext()) {
-        QRegularExpressionMatch match = it.next();
+        const QRegularExpressionMatch match = it.next();
         const ProgramData programData = processProgram(match, url, channelId, !it.hasNext());
         if (!programs.empty()) {
+            ProgramData &previousProgamData = programs.last();
             // sometimes, there can be multiple programs for the same time (e.g. different news per region of a local channel)
             // show this as alternative in the title
-            ProgramData &previousProgamData = programs.last();
-            if (programData.m_startTime < previousProgamData.m_stopTime) {
+            if (programData.m_startTime == previousProgamData.m_startTime) {
                 previousProgamData.m_title += QStringLiteral(" / ") + programData.m_title;
+            }
+            // it can happen that the stop time is incorrect (i.e. after the next program has already started)
+            // overwrite stop time with start time of next program
+            else if (programData.m_startTime < previousProgamData.m_stopTime) {
+                previousProgamData.m_stopTime = programData.m_startTime;
             } else {
                 programs.push_back(programData);
             }
@@ -227,8 +231,6 @@ QVector<ProgramData> TvSpielfilmFetcher::processChannel(const QString &infoTable
             programs.push_back(programData);
         }
     }
-
-    return programs;
 }
 
 ProgramData TvSpielfilmFetcher::processProgram(const QRegularExpressionMatch &programMatch, const QString &url, const ChannelId &channelId, bool isLast)
@@ -268,15 +270,114 @@ ProgramData TvSpielfilmFetcher::processProgram(const QRegularExpressionMatch &pr
 
 QString TvSpielfilmFetcher::processDescription(const QString &descriptionPage, const QString &url)
 {
-    static QRegularExpression reDescription(QStringLiteral("<section class=\\\"broadcast-detail__description\\\">.*?<p>(.*?)</p>"));
-    reDescription.setPatternOptions(QRegularExpression::DotMatchesEverythingOption);
-    QRegularExpressionMatch match = reDescription.match(descriptionPage);
-    if (match.hasMatch()) {
-        return match.captured(1);
-    } else {
+    QString description;
+
+    // description
+    {
+        static const QRegularExpression re(QStringLiteral("<section class=\\\"broadcast-detail__description\\\">.*?<p>(.*?)</p>"),
+                                           QRegularExpression::DotMatchesEverythingOption);
+        const QRegularExpressionMatch match = re.match(descriptionPage);
+        if (match.hasMatch()) {
+            description += match.captured(1) + QStringLiteral("<br>");
+        }
+    }
+
+    // original title
+    {
+        static const QRegularExpression re(QStringLiteral("<dt>Originaltitel:</dt>\\s*<dd>(.*?)</dd>"), QRegularExpression::DotMatchesEverythingOption);
+        const QRegularExpressionMatch match = re.match(descriptionPage);
+        if (match.hasMatch()) {
+            if (!description.isEmpty()) {
+                description += QStringLiteral("<br>");
+            }
+            description += i18n("Original title: ") + match.captured(1);
+        }
+    }
+
+    // country
+    {
+        static const QRegularExpression re(QStringLiteral("<dt>Land:</dt>\\s*<dd>(.*?)</dd>"), QRegularExpression::DotMatchesEverythingOption);
+        const QRegularExpressionMatch match = re.match(descriptionPage);
+        if (match.hasMatch()) {
+            if (!description.isEmpty()) {
+                description += QStringLiteral("<br>");
+            }
+            description += i18n("Country: ") + match.captured(1);
+        }
+    }
+
+    // year
+    {
+        static const QRegularExpression re(QStringLiteral("<dt>Jahr:</dt>\\s*<dd>(.*?)</dd>"), QRegularExpression::DotMatchesEverythingOption);
+        const QRegularExpressionMatch match = re.match(descriptionPage);
+        if (match.hasMatch()) {
+            if (!description.isEmpty()) {
+                description += QStringLiteral("<br>");
+            }
+            description += i18n("Year: ") + match.captured(1);
+        }
+    }
+
+    // duration
+    {
+        static const QRegularExpression re(QStringLiteral("<dt>Länge:</dt>\\s*<dd>(.*?)</dd>"), QRegularExpression::DotMatchesEverythingOption);
+        const QRegularExpressionMatch match = re.match(descriptionPage);
+        if (match.hasMatch()) {
+            if (!description.isEmpty()) {
+                description += QStringLiteral("<br>");
+            }
+            description += i18n("Duration: ") + match.captured(1);
+        }
+    }
+
+    // FSK
+    {
+        static const QRegularExpression re(QStringLiteral("<dt>FSK:</dt>\\s*<dd>(.*?)</dd>"), QRegularExpression::DotMatchesEverythingOption);
+        const QRegularExpressionMatch match = re.match(descriptionPage);
+        if (match.hasMatch()) {
+            if (!description.isEmpty()) {
+                description += QStringLiteral("<br>");
+            }
+            description += i18n("FSK: ") + match.captured(1);
+        }
+    }
+
+    // cast + crew
+    {
+        static const QRegularExpression reActors(QStringLiteral("<dl class=\\\"actors\\\"(.*?)</dl>"), QRegularExpression::DotMatchesEverythingOption);
+        const QRegularExpressionMatch actorsMatch = reActors.match(descriptionPage);
+        if (actorsMatch.hasMatch()) {
+            if (!description.isEmpty()) {
+                description += QStringLiteral("<br>");
+            }
+
+            static const QRegularExpression re(
+                QStringLiteral("<dt.*?>(.*?)</dt>.*?<dd.*?>\\s*(<a\\s*href=\\\".*?\\\"\\s*target=\\\".*?\\\"\\s*title=\\\".*?\\\">)?(.*?)(</a>)?\\s*</dd>"),
+                QRegularExpression::DotMatchesEverythingOption);
+            QRegularExpressionMatchIterator it = re.globalMatch(actorsMatch.captured(1));
+            while (it.hasNext()) {
+                const QRegularExpressionMatch match = it.next();
+                const QString role = match.captured(1).trimmed();
+                const QString name = match.captured(3).trimmed();
+                if (role == QStringLiteral("&nbsp;") && name == QStringLiteral("&nbsp;")) {
+                    // split cast and crew into 2 sections
+                    description += QStringLiteral("<br>");
+                } else {
+                    if (role.endsWith(QLatin1Char(':'))) {
+                        description += QStringLiteral("<br>") + role + QLatin1Char(' ') + name;
+                    } else {
+                        description += QStringLiteral("<br>") + role + QStringLiteral(" - ") + name;
+                    }
+                }
+            }
+        }
+    }
+
+    if (description.isEmpty()) {
         qWarning() << "Failed to parse program description from" << url;
     }
-    return QStringLiteral("");
+
+    return description;
 }
 
 bool TvSpielfilmFetcher::programExists(const ChannelId &channelId, const QDate &date)
